<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Categories\Edit\Audience;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use WriteTextAI\WriteTextAI\Model\Magento\Category as MagentoModel;

class Select extends Action
{
    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var MagentoModel
     */
    protected $magentoModel;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     * @param Premium $premium
     * @param MagentoModel $magentoModel
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager,
        Premium $premium,
        MagentoModel $magentoModel
    ) {
        $this->apiManager = $apiManager;
        $this->premium = $premium;
        $this->magentoModel = $magentoModel;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        if (!$this->premium->getIsPremium()) {
            $response->setData([
                'success' => false,
                'is_premium' => false,
                'message' => __('Premium is required to do this action.')
            ]);

            return $response;
        }
        
        try {
            $selected = $this->getRequest()->getParam('selected');
            $recordId = $this->getRequest()->getParam('category_id');
            $storeId = $this->getRequest()->getParam('store_id');

            $category = $this->magentoModel->getCategory($recordId, $storeId);

            $audience = $this->apiManager->selectAudience(
                $storeId,
                $recordId,
                $category->getName(),
                $selected,
                'Category'
            );
            
            $result = [
                'success' => true,
                'selected' => $audience['selected'],
                'values' => $audience['values']
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $response->setData($result);

        return $response;
    }
}
