<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Categories\Edit;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use Magento\Catalog\Model\CategoryFactory;
use WriteTextAI\WriteTextAI\Model\AiCategoryManager;
use WriteTextAI\WriteTextAI\Model\Categories\GeneratedText;
use WriteTextAI\WriteTextAI\Helper\Categories\Fields as FieldHelper;
use WriteTextAI\WriteTextAI\Model\Categories\MarkReview;
use WriteTextAI\WriteTextAI\Model\Categories\ReviewStatus;
use WriteTextAI\WriteTextAI\Helper\ReviewStatus as ReviewHelper;

class Transfer extends Action
{
    public const ADMIN_RESOURCE = 'WriteTextAI_WriteTextAI::transfer';

    /**
     * @var Premium
     */
    private $premium;

    /**
     * @var CategoryFactory
     */
    private $categoryFactory;

    /**
     * @var AiCategoryManager
     */
    private $aiCategoryManager;

    /**
     * @var GeneratedText
     */
    private $generatedText;

    /**
     * @var FieldHelper
     */
    private $fieldHelper;

    /**
     * @var MarkReview
     */
    private $markReview;

    /**
     * @var ReviewHelper
     */
    private $reviewHelper;

    /**
     * @var ReviewStatus
     */
    private $reviewStatus;

    /**
     * Constructor
     *
     * @param Context $context
     * @param Premium $premium
     * @param CategoryFactory $categoryFactory
     * @param AiCategoryManager $aiCategoryManager
     * @param GeneratedText $generatedText
     * @param FieldHelper $fieldHelper
     * @param MarkReview $markReview
     * @param ReviewHelper $reviewHelper
     * @param ReviewStatus $reviewStatus
     */
    public function __construct(
        Context $context,
        Premium $premium,
        CategoryFactory $categoryFactory,
        AiCategoryManager $aiCategoryManager,
        GeneratedText $generatedText,
        FieldHelper $fieldHelper,
        MarkReview $markReview,
        ReviewHelper $reviewHelper,
        ReviewStatus $reviewStatus
    ) {
        $this->premium = $premium;
        $this->categoryFactory = $categoryFactory;
        $this->aiCategoryManager = $aiCategoryManager;
        $this->generatedText = $generatedText;
        $this->fieldHelper = $fieldHelper;
        $this->markReview = $markReview;
        $this->reviewHelper = $reviewHelper;
        $this->reviewStatus = $reviewStatus;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        try {
            $params = $this->getRequest()->getParams();

            $mappingSettings = $this->fieldHelper->getMappingSettings();

            $categories = [];
            foreach ($params['selected_stores'] as $storeId) {
                $category = $this->categoryFactory->create()->setStoreId($storeId)->load($params['category_id']);

                foreach ($params['selected_fields'] as $field => $value) {
                    $category->setData($mappingSettings[$field], $value);
                }

                $category->save();
                $categories[$storeId] = $category;
            }

            $this->generatedText->saveCategoryText(
                $params['store_id'],
                $params['category_id'],
                $params['selected_fields']
            );
            
            $this->aiCategoryManager->saveDate($params['category_id'], $params['store_id'], 'transferred');

            $this->reviewStatus->updateReview(
                array_keys($params['selected_fields']),
                $params['category_id'],
                $params['selected_stores'],
                $categories
            );

            $useDefault = [];
            foreach ($mappingSettings as $field => $value) {
                $useDefault[$field] = $this->fieldHelper->isUsingDefaultValues(
                    $category,
                    $params['store_id'],
                    $value
                );
            }

            $result = [
                'success' => true,
                'reviewed' => $this->markReview->getReviewed($params['category_id'], $params['store_id']),
                'updatedReview' => $this->reviewHelper->getCategoryReviewStatus(
                    $params['category_id'],
                    $params['store_id']
                ),
                'use_default' => $useDefault
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }
        
        $result['is_premium'] = $this->premium->getIsPremium();
        $result['total_credits'] = $this->premium->getTotalCredits();

        $response->setData($result);

        return $response;
    }
}
