<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit\Keyword;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use WriteTextAI\WriteTextAI\Helper\Product as ProductHelper;
use Magento\Catalog\Helper\Output as OutputHelper;
use WriteTextAI\WriteTextAI\Model\Magento\Category as MagentoCategory;

class Select extends Action
{
    public const ADMIN_RESOURCE = 'WriteTextAI_WriteTextAI::add_keywords';

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var ProductHelper
     */
    protected $productHelper;

    /**
     * @var OutputHelper
     */
    protected $outputHelper;

    /**
     * @var MagentoCategory
     */
    protected $magentoCategory;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     * @param Premium $premium
     * @param ProductHelper $productHelper
     * @param OutputHelper $outputHelper
     * @param MagentoCategory $magentoCategory
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager,
        Premium $premium,
        ProductHelper $productHelper,
        OutputHelper $outputHelper,
        MagentoCategory $magentoCategory
    ) {
        $this->apiManager = $apiManager;
        $this->premium = $premium;
        $this->productHelper = $productHelper;
        $this->outputHelper = $outputHelper;
        $this->magentoCategory = $magentoCategory;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        if (!$this->premium->getIsPremium()) {
            $response->setData([
                'success' => false,
                'is_premium' => false,
                'total_credits' => $this->premium->getTotalCredits(),
                'message' => __('Premium is required to do this action.')
            ]);

            return $response;
        }
        
        try {
            $selected = $this->getRequest()->getParam('selected', []);
            $recordId = $this->getRequest()->getParam('product_id');
            $storeId = $this->getRequest()->getParam('store_id');
            $text = $this->getRequest()->getParam('text');
            $entityType = $this->getRequest()->getParam('entity_type');

            $entityName = $text;
            switch ($entityType) {
                case 'Product':
                    $product = $this->productHelper->getProduct($recordId, $storeId);
                    $entityName = $this->outputHelper->productAttribute(
                        $product,
                        $text,
                        'name'
                    );
                    break;
                case 'Category':
                    $category = $this->magentoCategory->getCategory($recordId, $storeId);
                    $entityName = $category->getName();
                    break;
            }
            
            $apiResponse = $this->apiManager->selectSemanticKeyword(
                $storeId,
                $recordId,
                $selected,
                $entityName,
                $entityType
            );

            $result = [
                'success' => true,
                'api_response' => $apiResponse
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $result['is_premium'] = $this->premium->getIsPremium();
        $result['total_credits'] = $this->premium->getTotalCredits();

        $response->setData($result);

        return $response;
    }
}
