<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit\Keyword;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\Api\Keywords as ApiKeywords;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use WriteTextAI\WriteTextAI\Model\ApiManager;

class SpellCheck extends Action
{
    public const ADMIN_RESOURCE = 'WriteTextAI_WriteTextAI::add_keywords';

    /**
     * @var ApiKeywords
     */
    protected $apiKeywords;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiKeywords $apiKeywords
     * @param Premium $premium
     * @param ApiManager $apiManager
     */
    public function __construct(
        Context $context,
        ApiKeywords $apiKeywords,
        Premium $premium,
        ApiManager $apiManager
    ) {
        $this->apiKeywords = $apiKeywords;
        $this->premium = $premium;
        $this->apiManager = $apiManager;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        if (!$this->premium->getIsPremium()) {
            $response->setData([
                'success' => false,
                'is_premium' => false,
                'total_credits' => $this->premium->getTotalCredits(),
                'message' => __('Premium is required to do this action.')
            ]);

            return $response;
        }
        
        try {
            $oldKeyword = $this->getRequest()->getParam('old_keyword');
            $newKeyword = $this->getRequest()->getParam('new_keyword');
            $keywords = $this->getRequest()->getParam('keywords');
            $productId = $this->getRequest()->getParam('product_id');
            $storeId = $this->getRequest()->getParam('store_id');
            $location = $this->getRequest()->getParam('location');
            $entityType = $this->getRequest()->getParam('entity_type');

            $this->apiKeywords->updateKeyword($storeId, $productId, $oldKeyword, $newKeyword, $location, $entityType);

            $newKeywords = array_map(function ($keyword) use ($oldKeyword, $newKeyword) {
                return $keyword === $oldKeyword ? $newKeyword : $keyword;
            }, $keywords);
            
            $semanticKeywords = $this->apiManager->getSemanticKeywords(
                $storeId,
                $productId,
                $newKeywords,
                'false',
                $entityType
            );
    
            $texts = $semanticKeywords['texts'];
            $textIndex = array_search($newKeyword, array_column($texts, 'text'));
            $data = [
                'text' => $newKeyword,
                'values' => $texts[$textIndex]['values'] ?? []
            ];
    
            $keywordsTexts = array_column($texts, 'text');
            if (!empty($keywordsTexts)) {
                $audience = $this->apiManager->getSuggestedAudience(
                    $storeId,
                    $productId,
                    $keywordsTexts,
                    'false',
                    $entityType
                );
            }
            
            $result = [
                'success' => true,
                'data' => $data,
                'selected_audience' => $audience['selected'] ?? [],
                'suggested_audience' => $audience['values'] ?? []
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage(),
                'trace' => $e->getTraceAsString()
            ];
        }

        $result['is_premium'] = $this->premium->getIsPremium();
        $result['total_credits'] = $this->premium->getTotalCredits();

        $response->setData($result);

        return $response;
    }
}
