<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\ViewModel;

use WriteTextAI\WriteTextAI\Model\ApiManager;

class Premium implements \Magento\Framework\View\Element\Block\ArgumentInterface
{
    public const UNLOCK_PREMIUM_URL = 'https://platform.writetext.ai/premium';

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var bool
     */
    protected $isPremium;

    /**
     * @var int
     */
    protected $totalCredits = 0;

    /**
     * @var bool
     */
    protected $freePremiumCredits = false;

    /**
     * @var bool
     */
    protected $freeCreditsAlreadyPremium = false;

    /**
     * @var array
     */
    protected $account = [];

    /**
     * Constructor
     *
     * @param ApiManager $apiManager
     */
    public function __construct(
        ApiManager $apiManager
    ) {
        $this->apiManager = $apiManager;
    }

    /**
     * Get is premium
     *
     * @return bool
     */
    public function isPremium()
    {
        if ($this->isPremium !== null) {
            return $this->isPremium;
        }

        return $this->getIsPremium();
    }

    /**
     * Set is premium
     *
     * @param bool $isPremium
     * @return void
     */
    protected function setIsPremium($isPremium)
    {
        $this->isPremium = $isPremium;
    }

    /**
     * Set freePremiumCredits
     *
     * @param bool $freePremiumCredits
     * @return void
     */
    public function setFreePremiumCredits($freePremiumCredits)
    {
        $this->freePremiumCredits = $freePremiumCredits;
    }

    /**
     * Set freeCreditsAlreadyPremium
     *
     * @param bool $freeCreditsAlreadyPremium
     * @return void
     */
    public function setFreeCreditsAlreadyPremium($freeCreditsAlreadyPremium)
    {
        $this->freeCreditsAlreadyPremium = $freeCreditsAlreadyPremium;
    }
    
    /**
     * Is premium
     *
     * @return bool
     */
    public function getIsPremium()
    {
        $credit = $this->apiManager->getCredit();
        $isPremium = isset($credit['isPremium']) ? (bool) $credit['isPremium'] : false;
        $this->setTotalCredits(isset($credit['availableCredits']) ? (int) $credit['availableCredits'] : 0);
        $this->setIsPremium($isPremium);
        $this->setFreePremiumCredits(
            isset($credit['freePremiumCredits']) ? (bool) $credit['freePremiumCredits'] : false
        );
        $this->setFreeCreditsAlreadyPremium(
            isset($credit['freeCreditsAlreadyPremium']) ? (bool) $credit['freeCreditsAlreadyPremium'] : false
        );
        return $isPremium;
    }

    /**
     * Get unclock premium url
     *
     * @return string
     */
    public function getUnlockPremiumUrl()
    {
        return self::UNLOCK_PREMIUM_URL;
    }

    /**
     * Get ads
     *
     * @return array
     */
    public function getAds()
    {
        return $this->apiManager->getAds();
    }

    /**
     * Set total credits
     *
     * @param int $totalCredits
     */
    public function setTotalCredits($totalCredits)
    {
        $this->totalCredits = $totalCredits;
    }

    /**
     * Get total credits
     *
     * @return int
     */
    public function getTotalCredits()
    {
        return $this->totalCredits;
    }

    /**
     * Get free premium credits
     *
     * @return bool
     */
    public function getFreePremiumCredits()
    {
        return $this->freePremiumCredits;
    }

    /**
     * Get free credits already premium
     *
     * @return bool
     */
    public function getFreeCreditsAlreadyPremium()
    {
        return $this->freeCreditsAlreadyPremium;
    }

    /**
     * Get freemium credits
     *
     * @return int
     */
    public function getFreemiumCredits()
    {
        if (!empty($this->account)) {
            $account = $this->account;
        } else {
            $account = $this->apiManager->getAccount();
        }

        return $account['account']['company']['freePremiumCredits'] ?? 110;
    }
}
