<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\ViewModel;

use WriteTextAI\WriteTextAI\Model\UserFieldsSelectedManager;

class UserFieldsSelected implements \Magento\Framework\View\Element\Block\ArgumentInterface
{
    /**
     * @var UserFieldsSelectedManager
     */
    protected $userFieldsSelectedManager;

    /**
     * @var array
     */
    protected $data = [];
    
    /**
     * Constructor
     *
     * @param UserFieldsSelectedManager $userFieldsSelectedManager
     */
    public function __construct(
        UserFieldsSelectedManager $userFieldsSelectedManager
    ) {
        $this->userFieldsSelectedManager = $userFieldsSelectedManager;
    }

    /**
     * Get settings
     *
     * @param string $name
     * @param bool $reset
     * @param string $entityType
     *
     * @return array
     */
    public function getUserFieldsSelected($name, $reset = false, $entityType = 'product')
    {
        if (isset($this->data[$name]) && !$reset) {
            return $this->data[$name];
        }

        return $this->getValue($name, $entityType);
    }

    /**
     * Set value
     *
     * @param string $name
     * @param array $value
     * @return void
     */
    public function setValue($name, $value)
    {
        $this->data[$name] = $value;
    }
    
    /**
     * Get value
     *
     * @param string $name
     * @param string $entityType
     *
     * @return array
     */
    protected function getValue($name, $entityType = 'product')
    {
        $value = $this->userFieldsSelectedManager->getUserFieldsSelected($name, $entityType);
        $this->setValue($name, $value);

        return $value;
    }
}
