/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'uiComponent',
    'underscore',
    'ko',
    'mage/translate',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/utils/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/edit/settings',
    'WriteTextAI_WriteTextAI/js/edit/keyword-analysis/competitors-ranking',
    'WriteTextAI_WriteTextAI/js/edit/keyword-analysis/my-ranking',
    'WriteTextAI_WriteTextAI/js/edit/keyword-analysis',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords/progress',
    'WriteTextAI_WriteTextAI/js/model/edit/generate/keyword-analysis',
    'WriteTextAI_WriteTextAI/js/model/edit/error-messages',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/model/edit/audience',
    'WriteTextAI_WriteTextAI/js/categories/model/edit/textfields',
    'WriteTextAI_WriteTextAI/js/utils/edit/textfields',
    'Magento_Ui/js/modal/modal'
], function (
    $,
    Component,
    _,
    ko,
    $t,
    product,
    keywords,
    keywordUtils,
    premium,
    settings,
    competitorsRankingComponent,
    myRankingComponent,
    keywordAnalysisComponent,
    keywordProgress,
    genKeywordAnalysis,
    errorMessagesModel,
    totalCredits,
    audience,
    textfields,
    textfieldsUtils
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            modalSelector: '.wtai-keyword-analysis',
            highlighted: true,
            entityType: 'category',
            targetKeywords: [],
            selectedSemanticKeywords: []
        },
        
        isHighlightedChange: false,
        keywordIdeasLoaded: false,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            this.bindModal();
            this.bindEvents();

            keywords.highlighted.subscribe(function (highlighted) {
                self.isHighlightedChange = true;
                self.highlighted(highlighted);
                self.isHighlightedChange = false;
            });

            self.highlighted.subscribe(function (highlighted) {
                if (!self.isHighlightedChange) {
                    keywords.highlighted(highlighted);
                    self.saveHighlighted(highlighted);
                }
            });

            keywords.targetKeywords.subscribe(function (targetKeywords) {
                self.isTargetKeywordsChange = true;
                self.targetKeywords(targetKeywords);
                self.isTargetKeywordsChange = false;
            });

            keywords.selectedSemanticKeywords.subscribe(
                function (selectedSemanticKeywords) {
                    self.isSelectedSemanticKeywordsChange = true;
                    self.selectedSemanticKeywords(selectedSemanticKeywords);
                    self.isSelectedSemanticKeywordsChange = false;
                }
            );

            self.selectedSemanticKeywords.subscribe(
                function (selectedSemanticKeywords) {
                    if (!self.isSelectedSemanticKeywordsChange) {
                        keywords.selectedSemanticKeywords(
                            selectedSemanticKeywords
                        );
                    }
                }
            );

            self.targetKeywords.subscribe(function (targetKeywords) {
                if (!self.isTargetKeywordsChange) {
                    keywords.targetKeywords(targetKeywords);
                }
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'highlighted',
                'targetKeywords',
                'selectedSemanticKeywords'
            ]);

            this.guideSteps = ko.computed(function () {
                return settings.guideSteps()
            });

            this.isPremium = ko.computed(function () {
                return premium.isPremium()
            });

            this.keywordsCount = ko.computed(function () {
                var keywords = this.targetKeywords().filter(function (keyword) {
                        return keywordUtils.keywordRemovable(keyword.text);
                }),
                    count = keywords ? keywords.length : 0;

                return count + '/' + this.rules.maxKeywords;
            }, this);

            this.semanticKeywordsCount = ko.computed(function () {
                var count = this.selectedSemanticKeywords()
                    ? this.selectedSemanticKeywords().length
                    : 0;

                return '(' + count + '/' + this.rules.maxSemanticKeywords + ')';
            }, this);

            this.analysisInProgress = keywordUtils.analysisInProgress();

            this.isSemanticMax = ko.computed(function () {
                var count = this.selectedSemanticKeywords()
                    ? this.selectedSemanticKeywords().length
                    : 0;

                return count >= this.rules.maxSemanticKeywords;
            }, this);

            return this;
        },

        /**
         * Bind events.
         */
        bindEvents: function () {
            $('.wtai-target-list .action-close').on('click', function (event) {
                var target = event.target,
                    parent = $(target).closest('li');

                parent.remove();
            });

            $('.wtai-semantic-list li').on('click', function () {
                $(this).toggleClass('wtai-selected');
            });
        },

        /**
         * Remove a keyword from the current product.
         */
        removeKeyword: function () {
            var url = self.deleteUrl,
                productId = product.productId(),
                storeId = product.storeId(),
                text = this.text,
                remainingKeywords = self
                    .targetKeywords()
                    .filter(function (keyword) {
                        return keyword.text !== text;
                    })
                    .map(function (keyword) {
                        return { text: keyword.text };
                    });
            
            if (!self.isPremium()) {
                return;
            }

            errorMessagesModel.messages([]);
            self.targetKeywords.remove(this);

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    product_id: productId,
                    store_id: storeId,
                    remaining_keywords: remainingKeywords,
                    entity_type: self.entityType.charAt(0).toUpperCase() + self.entityType.slice(1)
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showAnalysisLoader: true,
                success: function (response) {
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    
                    if (response.success) {
                        keywords.selectedSemanticKeywords(response.selected);
                        audience.selectedAudience(response.selected_audience.shift());
                        audience.suggestedAudience(response.suggested_audience);
                    } else {
                        errorMessagesModel.messages.push(response.message);
                        self.targetKeywords.push(this);
                    }
                }.bind(this)
            });
        },

        /**
         * Check if keyword is removable.
         *
         * @param {string} keyword
         * @returns {boolean}
         */
        keywordRemovable: function (keyword) {
            return keywordUtils.keywordRemovable(keyword);
        },

        /**
         * Get the keyword density for a given keyword.
         *
         * @param {string} keyword
         * @returns {ko.computed}
         */
        keywordDensity: function (keyword) {
            return ko.computed(function () {
                return self.calculateKeywordDensity(
                    keyword,
                    textfields.categoryDescription()
                );
            });
        },

        /**
         * Calculate the keyword density for a given keyword.
         *
         * @param {string} keyword
         * @param {string} categoryDescription
         *
         * @returns {string} Formatted keyword density
         */
        calculateKeywordDensity: function (
            keyword,
            categoryDescription
        ) {
            if (!categoryDescription) {
                return '';
            }

            var totalWordCount = textfieldsUtils.getWordCount(categoryDescription, false),
                keywordWordCount = textfieldsUtils.getWordCount(keyword, false),
                productDescKeywordCount = textfieldsUtils.countKeywordOccurences(categoryDescription, keyword),
                totalKeywordCount = productDescKeywordCount * keywordWordCount,
                keywordDensity = (totalKeywordCount / totalWordCount) * 100,
                parsedKeywordDensity = parseFloat(keywordDensity, 10).toFixed(2);

            return ' (' + parsedKeywordDensity + '%)';
        },

        /**
         * Check if keyword is selected.
         *
         * @param {string} keyword
         * @returns {boolean}
         */
        isSemanticSelected: function (keyword) {
            if (this.selectedSemanticKeywords()) {
                return this.selectedSemanticKeywords().indexOf(keyword) > -1;
            } else {
                return false;
            }
        },

        /**
         * Bind keyword analysis modal.
         */
        bindModal: function () {
            $(this.modalSelector).modal({
                type: 'slide',
                title: $t('Keyword Analysis'),
                modalClass: 'wtai-edit-modal wtai-edit-keyword-analysis',
                innerScroll: true,
                buttons: [],
                overlayClass: 'modals-overlay wtai-keyword-overlay',
                outerClickHandler: function () {
                    if (self.analysisInProgress() && !keywordProgress.progressFromOtherUserOngoing()) {
                        return;
                    }
                    $(self.modalSelector).modal('closeModal');
                    keywordProgress.status({});
                },
                closed: function () {
                    $('.wtai-keyword-overlay').remove();
                    keywords.errorMessage('');
                    keywords.successMessage('');
                },
                opened: function () {
                    $('.wtai-edit-keyword-analysis').trigger(
                        'contentUpdated'
                    );
                    genKeywordAnalysis.views(genKeywordAnalysis.views() + 1);
                }
            });
        },

        /**
         * Open keyword analysis modal.
         */
        openKeywordAnalysis: function () {
            if (!self.isPremium()) {
                return;
            }

            $(this.modalSelector).modal('openModal');

            if (!self.keywordIdeasLoaded) {
                competitorsRankingComponent.prototype.resetSortingAndFilters();
                myRankingComponent.prototype.resetSortingAndFilters();
                keywordAnalysisComponent.prototype.getData();
                self.keywordIdeasLoaded = true;
            }
        },

        /**
         * Reset keyword ideas loaded.
         */
        resetKeywordIdeasLoaded: function () {
            if (self.keywordIdeasLoaded) {
                self.keywordIdeasLoaded = false;
            }
        },

        /**
         * Save user settings for highlighted keywords.
         *
         * @param {boolean} highlighted
         * @returns {void}
         */
        saveHighlighted: function (highlighted) {
            var url = self.highlightUrl;
            var entityType = self.entityType;
            
            if (!self.isPremium()) {
                return;
            }

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    highlighted: highlighted,
                    entity_type: entityType
                },
                dataType: 'json',
                showWriteTextAILoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response.message);
                    }
                }
            });
        },

        /**
         * Select semantic keyword.
         *
         * @param {string} keyword
         * @returns {void}
         */
        selectKeyword: function (keyword) {
            var url = self.selectUrl,
                storeId = product.storeId(),
                productId = product.productId(),
                originalSelected = self.selectedSemanticKeywords().slice(),
                selected = self.selectedSemanticKeywords(),
                productName = product.productName();
                
            if (!self.isPremium()) {
                return;
            }

            errorMessagesModel.messages([]);

            /* check if keyword is already selected */
            if (selected.indexOf(keyword) > -1) {
                /* unselect keyword */
                selected.splice(selected.indexOf(keyword), 1);
            } else {
                /* check if max semantic keywords is reached */
                if (selected.length >= self.rules.maxSemanticKeywords) {
                    return;
                }

                /* select keyword */
                selected.push(keyword);
            }

            self.selectedSemanticKeywords(selected);

            $.ajax({
                url: url,
                type: 'POST',
                data: {
                    product_id: productId,
                    store_id: storeId,
                    text: productName,
                    selected: selected,
                    entity_type: self.entityType.charAt(0).toUpperCase() + self.entityType.slice(1)
                },
                dataType: 'json',
                showWriteTextAILoader: false,
                success: function (response) {
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }

                    if (!response.success) {
                        errorMessagesModel.messages.push(response.message);
                        self.selectedSemanticKeywords(originalSelected);
                    }
                }
            });
        }
    });
});
