/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/edit/mark-reviewed',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/error-messages',
    'WriteTextAI_WriteTextAI/js/categories/model/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt'

], function (
    $,
    ko,
    Component,
    _,
    markReviewed,
    product,
    errorMessagesModel,
    textfields,
    premium
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            reviewed: false
        },

        isMarkedReviewChange: false,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            $('.wtai-mark-reviewed > *').applyBindings(this);

            markReviewed.reviewed.subscribe(function (reviewed) {
                self.isMarkedReviewChange = true;
                self.reviewed(reviewed);
                self.isMarkedReviewChange = false;
            });

            self.reviewed.subscribe(function (reviewed) {
                if (!self.isMarkedReviewChange) {
                    self.markReviewed(reviewed);
                }
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'reviewed'
                ]);

            this.canReview = ko.computed(function () {
                var statuses = textfields.statuses(),
                    allowTransfer = this.isTransferAllowed,
                    transferStatuses = [
                        statuses.pageTitleTransferStatus,
                        statuses.pageDescriptionTransferStatus,
                        statuses.categoryDescriptionTransferStatus
                    ];

                var allFieldsTransferred = transferStatuses.every(function (status) {
                    return status === true;
                });

                return allowTransfer && !allFieldsTransferred;
            }, this);

            return this;
        },

        /**
         * Mark as reviewed
         *
         * @param {boolean} reviewed
         * @returns {void}
         */
        markReviewed: function (reviewed) {
            errorMessagesModel.messages([]);

            $.ajax({
                url: self.markReviewUrl,
                type: 'POST',
                data: {
                    reviewed: reviewed,
                    category_id: product.productId(),
                    store_id: product.storeId()
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                success: function (response) {
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                    }
                    if (!response.success) {
                        errorMessagesModel.messages.push(response.message);
                        self.isMarkedReviewChange = true;
                        self.reviewed(!reviewed);
                        self.isMarkedReviewChange = false;
                    } else {
                        markReviewed.reviewed(reviewed);
                    }
                }
            });
        }
    });
});
