/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/model/edit/keyword-filters',
    'WriteTextAI_WriteTextAI/js/utils/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords/progress',
    'WriteTextAI_WriteTextAI/js/utils/helpers',
    'WriteTextAI_WriteTextAI/js/model/edit/audience',
    'mage/translate'
], function (
    $,
    ko,
    Component,
    _,
    keywords,
    product,
    premium,
    prompt,
    totalCredits,
    keywordFilters,
    keywordUtils,
    keywordProgress,
    helperUtils,
    audience,
    $t
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            volumeSort: '',
            competitionSort: '',
            intentSort: '',
            rankSort: '',
            rankedKeywords: [],
            canShowMore: false,
            filterByCompetition: [$t('LOW'), $t('MEDIUM'), $t('HIGH')],
            defaultFilterByCompetition: [$t('LOW'), $t('MEDIUM'), $t('HIGH')],
            filterBySearchVolume: '{"min": 0, "max": null}',
            defaultFilterBySearchVolume: '{"min": 0, "max": null}',
            isFiltering: false
        },

        gettingData: null,
        isFilterChanged: true,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            $('.wtai-keyword-analysis-my-ranking > *').applyBindings(this);

            keywords.myRanking.subscribe(function (value) {
                if (value.keywords) {
                    this.rankedKeywords(value.keywords);
                } else {
                    this.rankedKeywords([]);
                }
            }, this);

            keywordFilters.rankedSortAndFilters.subscribe(function (value) {
                this.isFilterChanged = false;

                this.filterByCompetition(value.competition);
                this.filterBySearchVolume(`{"min": ${value.search_volume_minimum}, "max": ${value.search_volume_maximum}}`);

                switch (true) {
                    case value.volume_sort !== '':
                        this.sortBySearchVolume(value.volume_sort);
                        break;
                    case value.competition_sort !== '':
                        this.sortByDifficulty(value.competition_sort);
                        break;
                    case value.intent_sort !== '':
                        this.sortByIntent(value.intent_sort);
                        break;
                    case value.rank_sort !== '':
                        this.sortByRank(value.rank_sort);
                        break;
                    default:
                        this.sortByKeyword();
                        break;
                }

                this.isFilterChanged = true;
            }, this);

            this.rankedKeywords.subscribe(function (rankedKeywords) {
                if (rankedKeywords.length > 0) {
                    this.canShowMore(rankedKeywords.length > 5);
                    $(".wtai-keyword-analysis-my-ranking .wtai-keyword-row").hide();
                    $(".wtai-keyword-analysis-my-ranking .wtai-keyword-row").slice(0, 5).show();
                } else {
                    this.canShowMore(false);
                }
            }, this);

            this.filterBySearchVolume.subscribe(function (filterBySearchVolume) {
                var volumesObject = JSON.parse(filterBySearchVolume),
                    filterByCompetition = this.filterByCompetition() || [],
                    competitionValues = {
                        [$t('LOW')]: 1,
                        [$t('MEDIUM')]: 2,
                        [$t('HIGH')]: 3
                    };
                    
                filterByCompetition = filterByCompetition.map(function(competition) {
                    return competitionValues[competition] || competition;
                });

                self.saveFilter();

                if (!keywords.myRanking().keywords) {
                    return;
                }
                
                this.isFiltering(true);
                let filteredKeywords = keywords.myRanking().keywords.filter(function (keyword) {
                    return filterByCompetition.includes(competitionValues[keyword.competition]) && (
                        keyword.search_volume >= volumesObject.min && (
                            volumesObject.max === null ||
                            keyword.search_volume <= volumesObject.max
                        )
                    );
                }, this);

                filteredKeywords = this.setActiveFilters(filteredKeywords);

                this.rankedKeywords(filteredKeywords);
                this.isFiltering(false);
            }, this);

            this.filterByCompetition.subscribe(function (filterByCompetition) {
                var competitionValues = {
                    [$t('LOW')]: 1,
                    [$t('MEDIUM')]: 2,
                    [$t('HIGH')]: 3
                };

                filterByCompetition = filterByCompetition || [];
                filterByCompetition = filterByCompetition.map(function(competition) {
                    return competitionValues[competition] || competition;
                });

                self.saveFilter();
                
                if (!keywords.myRanking().keywords) {
                    return;
                }

                this.isFiltering(true);
                let filteredKeywords = keywords.myRanking().keywords.filter(function (keyword) {
                    var volumesObject = JSON.parse(this.filterBySearchVolume());

                    return filterByCompetition.includes(competitionValues[keyword.competition]) && (
                        keyword.search_volume >= volumesObject.min && (
                            volumesObject.max === null ||
                            keyword.search_volume <= volumesObject.max
                        )
                    );
                }, this);

                filteredKeywords = this.setActiveFilters(filteredKeywords);

                this.rankedKeywords(filteredKeywords);
                this.isFiltering(false);
            }, this);
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'volumeSort',
                    'competitionSort',
                    'intentSort',
                    'rankSort',
                    'rankedKeywords',
                    'canShowMore',
                    'filterByCompetition',
                    'filterBySearchVolume',
                    'isFiltering'
                ]);

            this.hasFilter = ko.computed(function () {
                var filterByCompetition = this.filterByCompetition() || [];

                return filterByCompetition.length !== this.defaultFilterByCompetition.length ||
                    this.filterBySearchVolume() !== this.defaultFilterBySearchVolume;
            }, this);

            this.hasSort = ko.computed(function () {
                return this.volumeSort() !== '' ||
                    this.competitionSort() !== '' ||
                    this.intentSort() !== '' ||
                    this.rankSort() !== '';
            }, this);

            this.targetKeywords = ko.computed(function () {
                return keywords.targetKeywords();
            });

            this.date = ko.computed(function () {
                return keywords.myRanking().date || '';
            });

            this.isKeywordsMax = ko.computed(function () {
                var keywords = this.targetKeywords().filter(function (keyword) {
                        return keywordUtils.keywordRemovable(keyword.text);
                }),
                    keywordsCount = keywords ? keywords.length : 0,
                    maxKeywords = (self && self.rules && self.rules.maxKeywords) ? self.rules.maxKeywords : 5;

                return keywordsCount >= maxKeywords;
            }, this);

            this.keywordsChangeOngoing = ko.computed(function () {
                return keywords.keywordsChangeOngoing();
            });

            this.isSectionDisabled = ko.computed(function () {
                return keywords.ideasSortFilterOngoing() ||
                    keywords.ideasShowMoreOngoing() ||
                    keywords.keywordsChangeOngoing() ||
                    keywords.manualKeywordsChangeOngoing() ||
                    keywordProgress.ideasLoading() ||
                    keywordProgress.selectedAndCompetitorLoading();
            });

            this.showRefresh = ko.computed(function () {
                return keywords.showRankedRefresh();
            }, this);

            this.noResultText = ko.computed(function () {
                return '<p>' + helperUtils.sprintf(
                    $t('This page is not ranking for any keywords as of %s'),
                    keywords.rankedDate()
                ) + '</p>'
                    + '<br>'
                    + '<p>' + $t('You may click the "Start AI-powered keyword analysis" button to refresh ranking data for the whole domain. We recommend doing this after a month has passed since your last request — any less than that may not return any significant results.') + '</p>';
            });

            return this;
        },

        /**
         * Set active filters.
         *
         * @param {Array} filteredKeywords
         * @returns {Array}
         */
        setActiveFilters: function (filteredKeywords) {
            if (this.competitionSort() !== '') {
                filteredKeywords = filteredKeywords.sort(function (a, b) {
                    if (a && b && a.competition && b.competition) {
                        var competitionValues = {
                            [$t('LOW')]: 1,
                            [$t('MEDIUM')]: 2,
                            [$t('HIGH')]: 3
                        };
    
                        var aValue = competitionValues[a.competition.toUpperCase()];
                        var bValue = competitionValues[b.competition.toUpperCase()];
    
                        if (self.competitionSort()) {
                            return aValue - bValue;
                        } else {
                            return bValue - aValue;
                        }
                    } else {
                        return 0;
                    }
                });
            } else if (this.volumeSort() !== '') {
                filteredKeywords = filteredKeywords.sort(function (a, b) {
                    if (self.volumeSort()) {
                        return a.search_volume - b.search_volume;
                    } else {
                        return b.search_volume - a.search_volume;
                    }
                });
            } else if (this.intentSort() !== '') {
                filteredKeywords = filteredKeywords.sort(function (a, b) {
                    if (self.intentSort()) {
                        return (a.intent || '').localeCompare(b.intent || '');
                    } else {
                        return (b.intent || '').localeCompare(a.intent || '');
                    }
                });
            } else if (this.rankSort() !== '') {
                filteredKeywords = filteredKeywords.sort(function (a, b) {
                    if (self.rankSort()) {
                        return a.rank_group - b.rank_group;
                    } else {
                        return b.rank_group - a.rank_group;
                    }
                });
            } else {
                filteredKeywords = filteredKeywords.sort(function (a, b) {
                    return a.rank_absolute - b.rank_absolute;
                });
            }

            return filteredKeywords;
        },

        /**
         * Check if keyword is removable.
         *
         * @param {String} keyword
         * @returns {Boolean}
         */
        isKeywordRemovable: function (keyword) {
            return keywordUtils.keywordRemovable(keyword, false);
        },

        /**
         * Reset filters
         */
        resetSortingAndFilters: function () {
            self.isFilterChanged = false;
            self.filterByCompetition([...self.defaultFilterByCompetition]);
            self.filterBySearchVolume(self.defaultFilterBySearchVolume);
            self.volumeSort('');
            self.competitionSort('');
            self.intentSort('');
            self.rankSort('');
            self.isFilterChanged = true;
        },

        /**
         * Show more keywords.
         */
        showMore: function () {
            $(".wtai-keyword-analysis-my-ranking .wtai-keyword-row:hidden").slice(0, 10).slideDown();
            if ($(".wtai-keyword-analysis-my-ranking .wtai-keyword-row:hidden").length === 0) {
                this.canShowMore(false);
            }
        },

        /**
         * Check if keyword is selected.
         *
         * @param {String} keyword
         * @returns {Boolean}
         */
        isKeywordSelected: function (keyword) {
            return keywordUtils.isKeywordSelected(keyword);
        },

        /**
         * Close modal.
         */
        closeModal: function () {
            $(self.modalSelector).modal('closeModal');
        },
        
        /**
         * Remove keyword from product.
         *
         * @param {Object} data
         * @param {Event} event
         *
         * @returns {void}
         */
        removeKeyword: function (data, event) {
            var element = event.currentTarget,
                parentElement = $(element).closest('tr');

            if (!premium.isPremium()) {
                self.closeModal();
                return;
            }

            $(parentElement).find('.action').addClass('wtai-loading');

            keywords.errorMessage('');
            
            let selectedKeyword = this.keyword;
            let keywordToRemove = ko.utils.arrayFirst(
                keywords.targetKeywords(),
                function (data) {
                    return (
                        data.text === selectedKeyword
                    );
                }
            );

            if (keywordToRemove) {
                keywords.targetKeywords.remove(keywordToRemove);
            }

            $.ajax({
                url: self.keywordDeleteUrl,
                type: 'POST',
                data: {
                    entity_type: self.entityType,
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    remaining_keywords: keywords.targetKeywords()
                        .map(function (keyword) {
                            return { text: keyword.text };
                        })
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showKeywordsChangeLoader: true,
                hideWriteTextTitleLoader: true,
                success: function (response) {
                    $(parentElement).find('.action').removeClass('wtai-loading');
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    
                    if (response.success) {
                        keywords.selectedSemanticKeywords(response.selected);
                        audience.selectedAudience(response.selected_audience.shift());
                        audience.suggestedAudience(response.suggested_audience);
                        keywords.targetRemoved(true);
                    } else {
                        keywords.errorMessage(response.message);
                        keywords.targetKeywords.push(keywordToRemove);
                    }
                }.bind(this)
            });
        },

        /**
         * Save product filter state
         */
        saveFilter: function () {
            var filterBySearchVolumeObj = JSON.parse(self.filterBySearchVolume());

            if (!self.isFilterChanged) {
                return;
            }
            
            if (self.gettingData !== null) {
                self.gettingData.abort();
            }

            self.gettingData = $.ajax({
                url: self.saveFilterUrl,
                type: 'POST',
                data: {
                    entity_type: self.entityType,
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    competition: self.filterByCompetition(),
                    search_volume_minimum: filterBySearchVolumeObj.min,
                    search_volume_maximum: filterBySearchVolumeObj.max,
                    competition_sort: self.competitionSort(),
                    volume_sort: self.volumeSort(),
                    rank_sort: self.rankSort(),
                    intent_sort: self.intentSort(),
                    type: 'ranked'
                },
                dataType: 'json',
                showWriteTextAILoader: false,
                showAnalysisLoader: false,
                success: function (response) {
                    if (!response.success) {
                        keywords.errorMessage(response.message);
                    }
                },
                error: function (xhr, status, error) {
                    if (status !== 'abort') {
                        keywords.errorMessage(error);
                    }
                }
            });
        },

        /**
         * Add keyword to product.
         *
         * @param {Object} data
         * @param {Event} event
         *
         * @returns {void}
         */
        addKeyword: function (data, event) {
            var targetKeywords = self
                    .targetKeywords()
                    .map((item) => item['text']),
                element = event.currentTarget,
                parentElement = $(element).closest('tr'),
                keywordData = {
                    text: this.keyword,
                    values: [],
                    temp: true,
                    search_volume: this.search_volume,
                    competition: this.competition,
                    rank_group: this.rank_group,
                    intent: this.intent
            };

            if (self.isKeywordsMax()) {
                return;
            }
                
            if (!premium.isPremium()) {
                self.closeModal();
                return;
            }
            
            keywords.errorMessage('');

            /* Validate product name */
            if (!keywordUtils.keywordRemovable(keywordData.text, false)) {
                if (self.entityType === 'Product') {
                    keywords.errorMessage($t('This keyword is the same as the product name and is already considered by default when generating text. Please add a different keyword.'));
                } else {
                    keywords.errorMessage($t('This keyword is the same as the category name and is already considered by default when generating text. Please add a different keyword.'));
                }
                return;
            }

            /* Validate keyword length */
            if (keywordData.text.length > self.rules.maxKeywordLength) {
                keywords.errorMessage('Target keyword is too long.');
                return;
            }

            /* Validate duplicate keyword */
            var isDuplicate = self.targetKeywords().find(keyword => {
                return keyword.text.toLowerCase() === keywordData.text.toLowerCase();
            });
            if (isDuplicate) {
                keywords.errorMessage($t('The entered keyword already exists.'));
                return;
            }

            $(parentElement).find('.action').addClass('wtai-loading');

            keywords.targetKeywords.push(keywordData);

            $.ajax({
                url: self.keywordAddUrl,
                type: 'POST',
                data: {
                    entity_type: self.entityType,
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    keywords: targetKeywords,
                    keyword: keywordData.text
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showKeywordsChangeLoader: true,
                hideWriteTextTitleLoader: true,
                success: function (response) {
                    var newKeyword = {};

                    $(parentElement).find('.action').removeClass('wtai-loading');

                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }

                    if (response.success) {
                        const oldKeyword = ko.utils.arrayFirst(
                            keywords.targetKeywords(),
                            function (data) {
                                return (
                                    data.text === response.data.text &&
                                    data.temp === true
                                );
                            }
                        );

                        if (oldKeyword) {
                            newKeyword = {...oldKeyword};
                            newKeyword.values = response.data.values;
                            keywords.targetKeywords.replace(oldKeyword, newKeyword);
                        } else {
                            keywords.targetKeywords.push(response.data);
                        }

                        audience.selectedAudience(response.selected_audience.shift());
                        audience.suggestedAudience(response.suggested_audience);
                        keywords.newTargetExists(true);
                    } else {
                        keywords.errorMessage(response.message);
                        keywords.targetKeywords.remove(function (data) {
                            return (
                                data.text === keywordData.text &&
                                data.temp === true
                            );
                        });
                    }
                }
            });

            return true;
        },

        /**
         * Sort keywords by rank.
         *
         * @param {Boolean} sort
         * @returns {void}
         */
        sortByRank: function (sort = null) {
            if (sort === null) {
                sort = !this.rankSort();
            }

            this.competitionSort('');
            this.volumeSort('');
            this.intentSort('');
            this.rankSort(sort);

            this.rankedKeywords(this.rankedKeywords().sort(function (a, b) {
                if (self.rankSort()) {
                    return a.rank_group - b.rank_group;
                } else {
                    return b.rank_group - a.rank_group;
                }
            }));

            self.saveFilter();
        },

        /**
         * Sort keywords by search volume.
         *
         * @param {Boolean} sort
         * @returns {void}
         */
        sortBySearchVolume: function (sort = null) {
            if (sort === null) {
                sort = !this.volumeSort();
            }

            this.competitionSort('');
            this.intentSort('');
            this.volumeSort(sort);
            this.rankSort('');

            this.rankedKeywords(this.rankedKeywords().sort(function (a, b) {
                if (self.volumeSort()) {
                    return a.search_volume - b.search_volume;
                } else {
                    return b.search_volume - a.search_volume;
                }
            }));
            
            self.saveFilter();
        },

        /**
         * Sort keywords by competition.
         *
         * @param {Boolean} sort
         * @returns {void}
         */
        sortByDifficulty: function (sort = null) {
            if (sort === null) {
                sort = !this.competitionSort();
            }

            this.competitionSort(sort);
            this.volumeSort('');
            this.intentSort('');
            this.rankSort('');

            this.rankedKeywords(this.rankedKeywords().sort(function (a, b) {
                if (a && b && a.competition && b.competition) {
                    var competitionValues = {
                        [$t('LOW')]: 1,
                        [$t('MEDIUM')]: 2,
                        [$t('HIGH')]: 3
                    };

                    var aValue = competitionValues[a.competition.toUpperCase()];
                    var bValue = competitionValues[b.competition.toUpperCase()];

                    if (self.competitionSort()) {
                        return aValue - bValue;
                    } else {
                        return bValue - aValue;
                    }
                } else {
                    return 0;
                }
            }));
            
            self.saveFilter();
        },

        /**
         * Sort keywords by intent.
         *
         * @param {Boolean} sort
         * @returns {void}
         */
        sortByIntent: function (sort = null) {
            if (sort === null) {
                sort = !this.intentSort();
            }

            this.competitionSort('');
            this.volumeSort('');
            this.intentSort(sort);
            this.rankSort('');

            this.rankedKeywords(this.rankedKeywords().sort(function (a, b) {
                if (self.intentSort()) {
                    return (a.intent || '').localeCompare(b.intent || '');
                } else {
                    return (b.intent || '').localeCompare(a.intent || '');
                }
            }));
            
            self.saveFilter();
        },

        /**
         * Sort keywords by keyword.
         */
        sortByKeyword: function () {
            this.competitionSort('');
            this.volumeSort('');
            this.intentSort('');
            this.rankSort('');

            this.rankedKeywords(this.rankedKeywords().sort(function (a, b) {
                return a.rank_absolute - b.rank_absolute;
            }));
            
            self.saveFilter();
        }
    });
});
