/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'mage/translate',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'Magento_Ui/js/modal/modal'
], function ($, ko, Component, $t, premium) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            setToFalseUrl: '',
            freemiumPopupModal: null,
            isFreePremium: false,
            isFreeCreditsAlreadyPremium: false,
            isFreePremiumChanged: false,
            notifText: '',
            subNotifText: '',
            headerText: '',
            selectors: {
                freemiumPopupModalSelector: '.wtai-freemium-popup-modal',
                freemiumPopupModalContainer: '.wtai-freemium-popup-container',
                freemiumPopupTriggerSelector: 'span.wtai-freemium',
                freemiumPopupOverlaySelector: '.wtai-freemium-popup-overlay',
            },
            classes: {
                modalClass: 'wtai-freemium-popup-modal',
                overlayClass: 'wtai-freemium-popup-overlay'
            }
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super().observe([
                'isFreePremium',
                'isFreeCreditsAlreadyPremium',
                'notifText',
                'subNotifText',
                'headerText'
            ]);

            this.openFreeCreditsModal = ko.computed(function () {
                var alreadyPremium = this.isFreeCreditsAlreadyPremium();
                var freePremium = this.isFreePremium();

                if (alreadyPremium) {
                    var str = $t("You've generated your first text within 24 hours of signup. As a special reward, we're giving you %sfree %f credits%e! Enjoy enhanced features and more powerful tools to make your content shine.");
                    str = str.replace('%s', '<strong>');
                    str = str.replace('%e', '</strong>');
                    str = str.replace('%f', this.freemiumCredits);

                    this.headerText($t("You earned %f free credits!").replace('%f', this.freemiumCredits));
                    this.notifText($t(str));
                    this.subNotifText("");
                }
                
                if (freePremium) {
                    var str = $t("You've generated your first text within 24 hours of signup. As a special reward, we're giving you %saccess to Premium and free %f credits%e! Enjoy enhanced features and more powerful tools to make your content shine.");
                    str = str.replace('%s', '<strong>');
                    str = str.replace('%e', '</strong>');
                    str = str.replace('%f', this.freemiumCredits);
                    
                    var subStr = $t("*Premium access is valid for one year or until you have used up your premium %f credits, whichever comes first.");
                    subStr = subStr.replace('%f', this.freemiumCredits);

                    this.headerText($t("You've unlocked premium!"));
                    this.notifText($t(str));
                    this.subNotifText($t(subStr));
                }
                
                return freePremium || alreadyPremium;
            }, this);

            return this;
        },
    
        /** @inheritdoc */
        initialize: function () {
            this._super();
    
            self = this;

            self.isFreePremium.subscribe(function (value) {
                if (!self.isFreePremiumChanged) {
                    premium.isFreePremium(value);
                }
            });

            self.isFreeCreditsAlreadyPremium.subscribe(function (value) {
                if (!self.isFreePremiumChanged) {
                    premium.isFreeCreditsAlreadyPremium(value);
                }
            });

            premium.isFreePremium.subscribe(function (value) {
                self.isFreePremiumChanged = true;
                self.isFreePremium(value);
                self.isFreePremiumChanged = false;
            });

            premium.isFreeCreditsAlreadyPremium.subscribe(function (value) {
                console.log('value', value);
                self.isFreePremiumChanged = true;
                self.isFreeCreditsAlreadyPremium(value);
                self.isFreePremiumChanged = false;
            });
    
        },
        
        /**
         * make sure history log is over edit page modal
         */
        adjustZIndex: function () {
            var mainModalZIndex = $('.wtai-edit-modal.wtai-edit-main').css('z-index'),
                overlayZIndex = mainModalZIndex + 2;
            
            return overlayZIndex;
        },

        /**
         * Set free premium credits to false
         *
         * @returns {void}
         */
        setFreePremiumCreditsToFalse: function () {
            self.isFreePremium(false);
            self.isFreeCreditsAlreadyPremium(false);
            
            $.ajax({
                url: self.setToFalseUrl,
                type: 'POST',
                data: {
                    'is_free_premium_credits': false
                },
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response);
                    }
                }
            });
        },

        /**
         * Close modal
         */
        closeModal: function () {
            self.setFreePremiumCreditsToFalse();
        }
    });
});
