/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'WriteTextAI_WriteTextAI/js/categories/model/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/edit/textfields',
    'WriteTextAI_WriteTextAI/js/model/signalr',
    'WriteTextAI_WriteTextAI/js/model/edit/gallery',
    'WriteTextAI_WriteTextAI/js/signalr',
    'WriteTextAI_WriteTextAI/js/categories/signalr',
    'WriteTextAI_WriteTextAI/js/categories/model/signalr'
], function (
    $,
    categoriesTextfields,
    textfields,
    signalRModel,
    gallery,
    signalRComponent,
    categoriesSignalRComponent,
    categoriesSignalRModel
) {
    'use strict';

    /**
     * Remove empty fields.
     *
     * @param {Array} fields
     * @returns {Array}
     */
    function removeEmptyFields(fields)
    {
        var selectedFields = fields || [];
        const fieldsCopy = [...selectedFields];
        fieldsCopy.forEach((field) => {
            switch (field) {
                case 'page_title':
                    if (textfields.mgPageTitle() === '') {
                        selectedFields.splice(
                            selectedFields.indexOf(field),
                            1
                        );
                    }
                    break;
                case 'page_description':
                    if (textfields.mgPageDescription() === '') {
                        selectedFields.splice(
                            selectedFields.indexOf(field),
                            1
                        );
                    }
                    break;
                case 'product_description':
                    if (textfields.mgProductDescription() === '') {
                        selectedFields.splice(
                            selectedFields.indexOf(field),
                            1
                        );
                    }
                    break;
                case 'short_product_description':
                    if (textfields.mgProductShortDescription() === '') {
                        selectedFields.splice(
                            selectedFields.indexOf(field),
                            1
                        );
                    }
                    break;
                case 'open_graph':
                    if (textfields.mgOpenGraph() === '') {
                        selectedFields.splice(
                            selectedFields.indexOf(field),
                            1
                        );
                    }
                    break;
            }
        });

        return selectedFields;
    }

    /**
     * Remove images with empty alt.
     *
     * @param {Array} selectedImages
     * @returns {Array}
     */
    function removeImageWithEmptyAlt(selectedImages)
    {
        var images = gallery.images();
        var selectedImages = selectedImages;

        try {
            images.forEach((image) => {
                if (image.alt === '') {
                    selectedImages.splice(
                        selectedImages.indexOf(image.id),
                        1
                    );
                }
            });
        } catch (e) {
            return selectedImages;
        }

        return selectedImages;
    }

    /**
     * Clear text fields.
     *
     * @param {Array} selectedFields
     * @param {Array} selectedImages
     *
     * @returns {void}
     */
    function clearFields(selectedFields, selectedImages)
    {
        signalRComponent.prototype.hideMessage();
        signalRModel.errorFields([]);
        signalRModel.errorMessage('');
        signalRModel.selectedFields(selectedFields);
        signalRModel.imagesGenerating(selectedImages);
        signalRModel.imagesGenerated([]);
        signalRModel.fieldsGenerated({
            page_title: null,
            page_description: null,
            product_description: null,
            excerpt: null,
            open_graph: null
        });
        if (Array.isArray(selectedFields)) {
            selectedFields.forEach((field) => {
                switch (field) {
                    case 'page_title':
                        textfields.pageTitle('');
                        signalRModel.fieldsGenerated()['page_title'] = false;
                        $('#wtaiPageTitleMask').removeClass('wtai-generated');
                        break;

                    case 'page_description':
                        textfields.pageDescription('');
                        signalRModel.fieldsGenerated()[
                            'page_description'
                        ] = false;
                        $('#wtaiPageDescMask').removeClass('wtai-generated');
                        break;

                    case 'product_description':
                        textfields.productDescription('');
                        signalRModel.fieldsGenerated()[
                            'product_description'
                        ] = false;
                        $('#wtaiProductDescMask + .mce-tinymce').removeClass('wtai-generated');
                        $('#wtaiProductDescMask + .tox-tinymce').removeClass('wtai-generated');
                        break;

                    case 'short_product_description':
                        textfields.productShortDescription('');
                        signalRModel.fieldsGenerated()['excerpt'] = false;
                        $('#wtaiExcerptMask + .mce-tinymce').removeClass('wtai-generated');
                        $('#wtaiExcerptMask + .tox-tinymce').removeClass('wtai-generated');
                        break;

                    case 'open_graph':
                        textfields.openGraph('');
                        signalRModel.fieldsGenerated()['open_graph'] = false;
                        $('#wtaiOpenGraphMask').removeClass('wtai-generated');
                        break;
                }
            });
        }
        /** Ensure this is set last to prevent premature notification display. */
        signalRModel.generating(true);
    }

    /**
     * Clear text fields.
     *
     * @param {Array} selectedFields
     * @param {Boolean} rewrite
     *
     * @returns {void}
     */
    function clearCategoryFields(selectedFields, rewrite)
    {
        categoriesSignalRComponent.prototype.hideMessage();
        categoriesSignalRModel.errorFields([]);
        categoriesSignalRModel.errorMessage('');
        categoriesSignalRModel.selectedFields(selectedFields);
        categoriesSignalRModel.fieldsGenerated({
            page_title: null,
            page_description: null,
            category_description: null
        });
        if (Array.isArray(selectedFields)) {
            selectedFields.forEach((field) => {
                switch (field) {
                    case 'page_title':
                        if (!rewrite || (rewrite && categoriesTextfields.mgPageTitle() !== '')) {
                            categoriesTextfields.pageTitle('');
                            $('#wtaiPageTitleMask').removeClass('wtai-generated');
                        }
                        break;

                    case 'page_description':
                        if (!rewrite || (rewrite && categoriesTextfields.mgPageDescription() !== '')) {
                            categoriesTextfields.pageDescription('');
                            $('#wtaiPageDescMask').removeClass('wtai-generated');
                        }
                        break;

                    case 'category_description':
                        if (!rewrite || (rewrite && categoriesTextfields.mgCategoryDescription() !== '')) {
                            categoriesTextfields.categoryDescription('');
                            $('#wtaiCategoryDescMask + .mce-tinymce').removeClass('wtai-generated');
                            $('#wtaiCategoryDescMask + .tox-tinymce').removeClass('wtai-generated');
                        }
                        break;
                }
            });
        }
        /** Ensure this is set last to prevent premature notification display. */
        categoriesSignalRModel.generating(true);
    }

    return {
        removeEmptyFields: removeEmptyFields,
        clearFields: clearFields,
        removeImageWithEmptyAlt: removeImageWithEmptyAlt,
        clearCategoryFields: clearCategoryFields
    }
});